@@ -50,6 +50,7 @@
 #include "udisksthreadedjob.h"
 #include "udisksata.h"
 #include "udiskslinuxdevice.h"
+#include "udisksconfigmanager.h"
 
 /**
  * SECTION:udiskslinuxdriveata
@@ -434,7 +435,8 @@ selftest_status_to_string (SkSmartSelfTestExecutionStatus status)
   return ret;
 }
 
-static gboolean get_pm_state (UDisksLinuxDevice *device, GError **error, guchar *count)
+static gboolean
+get_pm_state (UDisksLinuxDevice *device, GError **error, guchar *count)
 {
   int fd;
   gboolean rc = FALSE;
@@ -446,7 +448,7 @@ static gboolean get_pm_state (UDisksLinuxDevice *device, GError **error, guchar
   if (fd == -1)
     {
       g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                   "Error opening device file %s: %m",
+                   "Error opening device file %s while getting PM state: %m",
                    g_udev_device_get_device_file (device->udev_device));
       goto out;
     }
@@ -485,13 +487,19 @@ static gboolean update_io_stats (UDisksLinuxDriveAta *drive, UDisksLinuxDevice *
     }
   else
     {
-      fscanf (statf, "%lu %*u %*u %*u %lu", &drive_read, &drive_write);
+      if (fscanf (statf, "%lu %*u %*u %*u %lu", &drive_read, &drive_write) != 2)
+        {
+          udisks_warning ("Failed to read %s\n", statpath);
+        }
+      else
+        {
+          noio = drive_read == drive->drive_read && drive_write == drive->drive_write;
+          udisks_debug ("drive_read=%lu, drive_write=%lu, old_drive_read=%lu, old_drive_write=%lu\n",
+                        drive_read, drive_write, drive->drive_read, drive->drive_write);
+          drive->drive_read = drive_read;
+          drive->drive_write = drive_write;
+        }
       fclose (statf);
-      noio = drive_read == drive->drive_read && drive_write == drive->drive_write;
-      udisks_debug ("drive_read=%lu, drive_write=%lu, old_drive_read=%lu, old_drive_write=%lu\n",
-                    drive_read, drive_write, drive->drive_read, drive->drive_write);
-      drive->drive_read = drive_read;
-      drive->drive_write = drive_write;
     }
   return noio;
 }
@@ -590,7 +598,7 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
     {
       guchar count;
       gboolean noio = FALSE;
-      if (!get_pm_state(device, error, &count))
+      if (!get_pm_state (device, error, &count))
         goto out;
       awake = count == 0xFF || count == 0x80;
       if (drive->standby_enabled)
@@ -910,8 +918,8 @@ handle_smart_selftest_abort (UDisksDriveAta        *_drive,
                              GDBusMethodInvocation *invocation,
                              GVariant              *options)
 {
-  UDisksLinuxDriveObject  *object;
-  UDisksLinuxBlockObject *block_object;
+  UDisksLinuxDriveObject *object;
+  UDisksLinuxBlockObject *block_object = NULL;
   UDisksDaemon *daemon;
   UDisksLinuxDriveAta *drive = UDISKS_LINUX_DRIVE_ATA (_drive);
   GError *error;
@@ -999,6 +1007,7 @@ handle_smart_selftest_abort (UDisksDriveAta        *_drive,
 
  out:
   g_clear_object (&object);
+  g_clear_object (&block_object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
@@ -1133,11 +1142,10 @@ handle_smart_selftest_start (UDisksDriveAta        *_drive,
                              GVariant              *options)
 {
   UDisksLinuxDriveObject  *object;
-  UDisksLinuxBlockObject *block_object;
+  UDisksLinuxBlockObject *block_object = NULL;
   UDisksDaemon *daemon;
   UDisksLinuxDriveAta *drive = UDISKS_LINUX_DRIVE_ATA (_drive);
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error;
 
   error = NULL;
@@ -1174,8 +1182,6 @@ handle_smart_selftest_start (UDisksDriveAta        *_drive,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1240,11 +1246,85 @@ handle_smart_selftest_start (UDisksDriveAta        *_drive,
 
  out:
   g_clear_object (&object);
+  g_clear_object (&block_object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+/**
+ * udisks_linux_drive_ata_get_pm_state:
+ * @drive: A #UDisksLinuxDriveAta.
+ * @error: Return location for error.
+ * @pm_state: Return location for the current power state value.
+ *
+ * Get the current power mode state.
+ *
+ * The format of @pm_state is the result obtained from sending the
+ * ATA command `CHECK POWER MODE` to the drive.
+ *
+ * Known values include:
+ *  - `0x00`: Device is in PM2: Standby state.
+ *  - `0x40`: Device is in the PM0: Active state, the NV Cache power mode is enabled, and the spindle is spun down or spinning down.
+ *  - `0x41`: Device is in the PM0: Active state, the NV Cache power mode is enabled, and the spindle is spun up or spinning up.
+ *  - `0x80`: Device is in PM1: Idle state.
+ *  - `0xff`: Device is in the PM0: Active state or PM1: Idle State.
+ *
+ * Typically user interfaces will report "Drive is spun down" if @pm_state is
+ * 0x00 and "Drive is spun up" otherwise.
+ *
+ * Returns: %TRUE if the operation succeeded, %FALSE if @error is set.
+ */
+gboolean
+udisks_linux_drive_ata_get_pm_state (UDisksLinuxDriveAta  *drive,
+                                     GError              **error,
+                                     guchar               *pm_state)
+{
+  UDisksLinuxDriveObject *object;
+  UDisksLinuxDevice *device = NULL;
+  gboolean ret = FALSE;
+
+  object = udisks_daemon_util_dup_object (drive, error);
+  if (object == NULL)
+    goto out;
+
+  if (!udisks_drive_ata_get_pm_supported (UDISKS_DRIVE_ATA (drive)) ||
+      !udisks_drive_ata_get_pm_enabled (UDISKS_DRIVE_ATA (drive)))
+    {
+      g_set_error_literal (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                           "PM is not supported or enabled");
+      goto out;
+    }
+
+  /* If a secure erase is in progress, the CHECK POWER command would be queued
+   * until the erase has been completed (can easily take hours).
+   */
+  if (drive->secure_erase_in_progress)
+    {
+      g_set_error_literal (error, UDISKS_ERROR, UDISKS_ERROR_DEVICE_BUSY,
+                           "A secure erase is in progress");
+      goto out;
+    }
+
+  /* TODO: some SSD controllers may block for considerable time when trimming large amount of blocks */
+
+  device = udisks_linux_drive_object_get_device (object, TRUE /* get_hw */);
+  if (device == NULL)
+    {
+      g_set_error_literal (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                           "No udev device");
+      goto out;
+    }
+
+  ret = get_pm_state (device, error, pm_state);
+
+ out:
+  g_clear_object (&device);
+  g_clear_object (&object);
+
+  return ret;
+}
+
 static gboolean
 handle_pm_get_state (UDisksDriveAta        *_drive,
                      GDBusMethodInvocation *invocation,
@@ -1253,7 +1333,6 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
   UDisksLinuxDriveAta *drive = UDISKS_LINUX_DRIVE_ATA (_drive);
   UDisksLinuxDriveObject  *object = NULL;
   UDisksDaemon *daemon;
-  UDisksLinuxDevice *device = NULL;
   GError *error = NULL;
   const gchar *message;
   const gchar *action_id;
@@ -1266,27 +1345,6 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
       goto out;
     }
 
-  daemon = udisks_linux_drive_object_get_daemon (object);
-
-  if (!udisks_drive_ata_get_pm_supported (UDISKS_DRIVE_ATA (drive)) ||
-      !udisks_drive_ata_get_pm_enabled (UDISKS_DRIVE_ATA (drive)))
-    {
-      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "PM is not supported or enabled");
-      goto out;
-    }
-
-  /* If a secure erase is in progress, the CHECK POWER command would be queued
-   * until the erase has been completed (can easily take hours). So just return
-   * 0xff which is active/idle...
-   */
-  if (drive->secure_erase_in_progress)
-    {
-      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_DEVICE_BUSY,
-                                             "A secure erase is in progress");
-      goto out;
-    }
-
   /* Translators: Shown in authentication dialog when the user
    * requests the power state of a drive.
    *
@@ -1299,6 +1357,7 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
   /* TODO: maybe not check with polkit if this is OK (consider gnome-disks(1) polling all drives every few seconds) */
 
   /* Check that the user is authorized */
+  daemon = udisks_linux_drive_object_get_daemon (object);
   if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                     UDISKS_OBJECT (object),
                                                     action_id,
@@ -1307,20 +1366,12 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
                                                     invocation))
     goto out;
 
-  device = udisks_linux_drive_object_get_device (object, TRUE /* get_hw */);
-  if (device == NULL)
-    {
-      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "No udev device");
-      goto out;
-    }
-  if (get_pm_state (device, &error, &count))
+  if (udisks_linux_drive_ata_get_pm_state (drive, &error, &count))
     udisks_drive_ata_complete_pm_get_state (_drive, invocation, count);
   else
     g_dbus_method_invocation_take_error (invocation, error);
 
  out:
-  g_clear_object (&device);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
@@ -1382,8 +1433,6 @@ handle_pm_standby_wakeup (UDisksDriveAta        *_drive,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               NULL,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1431,7 +1480,7 @@ handle_pm_standby_wakeup (UDisksDriveAta        *_drive,
   if (fd == -1)
     {
       g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "Error opening device file %s: %m",
+                                             "Error opening device file %s while changing PM state: %m",
                                              g_udev_device_get_device_file (device->udev_device));
       goto out;
     }
@@ -1574,51 +1623,26 @@ static gpointer
 apply_configuration_thread_func (gpointer user_data)
 {
   ApplyConfData *data = user_data;
+  UDisksDaemon *daemon;
   const gchar *device_file = NULL;
   gint fd = -1;
   GError *error = NULL;
 
+  daemon = udisks_linux_drive_object_get_daemon (data->object);
   device_file = g_udev_device_get_device_file (data->device->udev_device);
 
-  udisks_notice ("Applying configuration from %s/udisks2/%s.conf to %s",
-                 PACKAGE_SYSCONF_DIR, udisks_drive_get_id (data->drive), device_file);
-
+  udisks_notice ("Applying configuration from %s/%s.conf to %s",
+                 udisks_config_manager_get_config_dir (udisks_daemon_get_config_manager (daemon)),
+                 udisks_drive_get_id (data->drive), device_file);
 
   /* Use O_RDRW instead of O_RDONLY to force a 'change' uevent so properties are updated */
   fd = open (device_file, O_RDWR|O_NONBLOCK);
   if (fd == -1)
     {
-      udisks_critical ("Error opening device file %s: %m", device_file);
+      udisks_critical ("Error opening device file %s while applying ATA configuration: %m", device_file);
       goto out;
     }
 
-  if (data->ata_pm_standby != -1)
-    {
-      /* ATA8: 7.18 IDLE - E3h, Non-Data */
-      UDisksAtaCommandInput input = {.command = 0xe3, .count = data->ata_pm_standby};
-      UDisksAtaCommandOutput output = {0};
-      if (!udisks_ata_send_command_sync (fd,
-                                         -1,
-                                         UDISKS_ATA_COMMAND_PROTOCOL_NONE,
-                                         &input,
-                                         &output,
-                                         &error))
-        {
-          udisks_critical ("Error sending ATA command IDLE (timeout=%d) to %s: %s (%s, %d)",
-                        data->ata_pm_standby, device_file,
-                        error->message, g_quark_to_string (error->domain), error->code);
-          g_clear_error (&error);
-        }
-      else
-        {
-          gchar *pretty = ata_pm_standby_to_string (data->ata_pm_standby);
-          udisks_notice ("Set standby timer to %s (value %d) on %s [%s]",
-                         pretty, data->ata_pm_standby, device_file, udisks_drive_get_id (data->drive));
-          g_free (pretty);
-          data->ata->standby_enabled = data->ata_pm_standby != 0;
-        }
-    }
-
   if (data->ata_apm_level != -1)
     {
       /* ATA8: 7.48 SET FEATURES - EFh, Non-Data
@@ -1650,6 +1674,33 @@ apply_configuration_thread_func (gpointer user_data)
         }
     }
 
+  if (data->ata_pm_standby != -1)
+    {
+      /* ATA8: 7.18 IDLE - E3h, Non-Data */
+      UDisksAtaCommandInput input = {.command = 0xe3, .count = data->ata_pm_standby};
+      UDisksAtaCommandOutput output = {0};
+      if (!udisks_ata_send_command_sync (fd,
+                                         -1,
+                                         UDISKS_ATA_COMMAND_PROTOCOL_NONE,
+                                         &input,
+                                         &output,
+                                         &error))
+        {
+          udisks_critical ("Error sending ATA command IDLE (timeout=%d) to %s: %s (%s, %d)",
+                        data->ata_pm_standby, device_file,
+                        error->message, g_quark_to_string (error->domain), error->code);
+          g_clear_error (&error);
+        }
+      else
+        {
+          gchar *pretty = ata_pm_standby_to_string (data->ata_pm_standby);
+          udisks_notice ("Set standby timer to %s (value %d) on %s [%s]",
+                         pretty, data->ata_pm_standby, device_file, udisks_drive_get_id (data->drive));
+          g_free (pretty);
+          data->ata->standby_enabled = data->ata_pm_standby != 0;
+        }
+    }
+
   if (data->ata_aam_level != -1)
     {
       /* ATA8: 7.48 SET FEATURES - EFh, Non-Data
@@ -1657,7 +1708,7 @@ apply_configuration_thread_func (gpointer user_data)
        */
       UDisksAtaCommandInput input = {.command = 0xef, .feature = 0x42, .count = data->ata_aam_level};
       UDisksAtaCommandOutput output = {0};
-      if (data->ata_apm_level == 0xff)
+      if (data->ata_aam_level == 0xff)
         {
           input.feature = 0xc2;
           input.count = 0x00;
@@ -1896,6 +1947,12 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta  *drive,
   if (object == NULL)
     goto out;
   _drive = udisks_object_peek_drive (UDISKS_OBJECT (object));
+  if (_drive == NULL)
+    {
+      g_set_error (&local_error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Failed to get Drive interface for object");
+      goto out;
+    }
 
   block_object = udisks_linux_drive_object_get_block (object, FALSE);
   if (block_object == NULL)
@@ -1928,7 +1985,7 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta  *drive,
   if (fd == -1)
     {
       g_set_error (&local_error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                   "Error opening device file %s: %m",
+                   "Error opening device file %s for secure erase: %m",
                    device_file);
       goto out;
     }
@@ -2146,7 +2203,7 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta  *drive,
     {
       udisks_notice ("Error securely erasing %s (%s): %s (%s, %d)",
                      device_file,
-                     udisks_drive_get_id (_drive),
+                     _drive ? udisks_drive_get_id (_drive) : "",
                      local_error->message, g_quark_to_string (local_error->domain), local_error->code);
     }
 
@@ -2193,9 +2250,7 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
   const gchar *message;
   const gchar *action_id;
   uid_t caller_uid;
-  gid_t caller_gid;
   gboolean enhanced = FALSE;
-  UDisksInhibitCookie *inhibit_cookie = NULL;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -2221,8 +2276,6 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2250,8 +2303,6 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
                                                     invocation))
     goto out;
 
-  inhibit_cookie = udisks_daemon_util_inhibit_system_sync (N_("Formatting Device"));
-
   if (!udisks_linux_drive_ata_secure_erase_sync (drive, caller_uid, enhanced, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2260,11 +2311,10 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
     }
 
   udisks_linux_block_object_reread_partition_table (UDISKS_LINUX_BLOCK_OBJECT (block_object));
-
-  udisks_drive_ata_complete_security_erase_unit (_drive, invocation);
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (block_object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
 
  out:
-  udisks_daemon_util_uninhibit_system_sync (inhibit_cookie);
   g_clear_object (&block_object);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
@@ -2288,7 +2338,6 @@ handle_smart_set_enabled (UDisksDriveAta        *_drive,
   const gchar *message;
   const gchar *action_id;
   uid_t caller_uid;
-  gid_t caller_gid;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -2314,8 +2363,6 @@ handle_smart_set_enabled (UDisksDriveAta        *_drive,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2366,7 +2413,7 @@ handle_smart_set_enabled (UDisksDriveAta        *_drive,
   if (fd == -1)
     {
       g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "Error opening device file %s: %m",
+                                             "Error opening device file %s while changing SMART status: %m",
                                              g_udev_device_get_device_file (device->udev_device));
       goto out;
     }
